#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

#TODOs:
# Reasoning mit Pellet
# Agent soll nur dann Anfrage stellen, wenn er noch keine Antwort auf die Anfrage hat
# soll aufhoeren, wenn er alle Informationen gefunden hat, z.B. durch Liste: targetList = [0,0,0,0,0], ist am Ziel, wenn targetList = [1,1,1,1,1]
# soll Mondkalender nicht aufrufen, wenn er keine Saatzeit findet
# soll die Informationen der annotierten Seiten bei Annotationsfehlern nicht beruecksichtigen

import sys
sys.path.append(sys.path[0] + "/ZSI")
sys.path.append(sys.path[0] +"/rdflib")

import vegi
import webService
import moon
import sparql
import datetime
import re
from time import *		
import ConfigParser
import string


			
	
def getMaxSeedingInterval(vegiObj, year):
	"""Sucht das grte Saatzeitintervall. Vergleicht, ob der Indoor- oder Outdoor-Saattermin frueher ist. Gibt den fruehesten und den spaetesten Saattermin als datetime.date-Objekt zurueck.
	"""
	if vegiObj.getSeedingTimeIndoor().beginGMonthDay != None:
		gMonthDay = vegiObj.getSeedingTimeIndoor().beginGMonthDay.strip("-").split("-")
		b1 = datetime.date(int(year),int(gMonthDay[0]),int(gMonthDay[1]))
		gMonthDay = vegiObj.getSeedingTimeIndoor().endGMonthDay.strip("-").split("-")
		e1 = datetime.date(int(year),int(gMonthDay[0]),int(gMonthDay[1]))
	else: 
		b1 = None
		e1 = None
	if vegiObj.getSeedingTimeOutdoor().beginGMonthDay != None:
		gMonthDay = vegiObj.getSeedingTimeOutdoor().beginGMonthDay.strip("-").split("-")
		b2 = datetime.date(int(year),int(gMonthDay[0]),int(gMonthDay[1]))
		gMonthDay = vegiObj.getSeedingTimeOutdoor().endGMonthDay.strip("-").split("-")
		e2 = datetime.date(int(year),int(gMonthDay[0]),int(gMonthDay[1]))
	else: 
		b2 = b1
		e2 = e1
	if b1 == None:
		b1=b2
		e1=e2
	if b1>b2:
		b1 = b2
	if e1<e2:
		e1 = e2
	return (b1,e1)
		
	
def goodSeedingDate(date, vegi, begin, end):
	"""entscheidet, ob date fuer das uebergebene Gemuese und den Saatzeitraum nach dem Mondkalender ein gutes Datum ist
	"""
	(day,month,year) = date.mcXsdDate.split("-");
	dateObj = datetime.date(int(year),int(month),int(day))
	if (dateObj < begin) or (dateObj > end):
		return False
	if len(vegi.getBestSeedingTimeMoonPhase()) > 0:
		if re.search("WaxingMoon$", vegi.getBestSeedingTimeMoonPhase()) != None:
			if not date.getWaxingMoon():
				return False
		else:
			if date.getWaxingMoon():
				return False
	if len(vegi.getBestSeedingTimeSignOfZodiac()) > 0:
		for sign in vegi.getBestSeedingTimeSignOfZodiac():
			try:
				if re.search(date.getSignOfZodiac().split("/")[-1],sign,re.I) != None:
					return True
			except IndexError:
				pass
	else:
		return True
	return False


def processWebService(ws, ontoDict,begin, end):
	"""fuehrt den Web Service aus und gibt ein MoonCalendar-Objekt zurueck fuer den gesamten Saatzeitraum.
	"""
	para = {}
	#die Parameter muessten eigentlich mit einer SPARQL-Abfrage ermittelt werden.
	para['month']=str(begin.month)
	para['year']=str(begin.year)
	if not ws.wsdl.processWebservice(para):
		return None
	ws.processXSLT(ws.wsdl.lastResponse)
	mk = moon.MoonCalendar(ontoDict,ws.rdfFile.name)	
	for mkday in mk.moonDaysList:
		mkday.setMcXsdDate()
		mkday.setMoonPhase()
		mkday.setMcXsdDate()
		mkday.setSignOfZodiac()
	month = begin.month + 1
	while month <= end.month:
		para['month']=str(month)
		month = month + 1
		if not ws.wsdl.processWebservice(para):
			return mk 
		ws.processXSLT(ws.wsdl.lastResponse)
		for mkday in mk.appendMoonDays(ws.rdfFile.name):
			mkday.setMcXsdDate()
			mkday.setMoonPhase()
			mkday.setMcXsdDate()
			mkday.setSignOfZodiac()
	return mk


def readInput():
	vegi = raw_input("Bitte ein Gemuese eingeben:  ").title()
	year = raw_input("Bitte Aussaatjahr eingeben [" + str(datetime.date.today().year) + "]:  ")
	print "Bitte etwas Geduld..."
	if year.strip() == "":
		year = str(datetime.date.today().year)
	return (vegi.strip(),year.strip())
	
	
def _kommaList(list):
	"""die einzelnen Elemente einer Liste werden mit Kommas getrennt ausgegen. Macht einen Zeilenumbruch nach < 40 Zeichen.
	"""
	r = ""
	l = 0
	for i in list:
		if r != "":
			r = r + ", "
			l = l + 2
			if l > 40:
				l = 0
				r = r + """
                                    |  """
		r = r + i
		l = l + len(i)
	return r


def _nonePrint(obj):
	"""Fehlerbehandlung: wenn ein Objekt None ist, gibt es keine Rueckgabe
	"""
	if obj == None:
		return ""
	return obj

def readConfig(configFile):
	"""Liest die Konfigurationsdatei, gibt 
	   1. Dict mit ontologien zurueck
	   2. Liste mit Webpages
	   3. Liste mit Webservices zurck
	"""
	config = ConfigParser.ConfigParser()
	config.read(configFile)
	onto = {}
	webpage = []
	webservice =[]
	sec = "ontologies"
	for option in config.options(sec):
		onto[option] = config.get(sec, option)
	sec = "webpages"
	for option in config.options(sec):
		webpage.append(config.get(sec, option))
	sec = "webservices"
	for option in config.options(sec):
		webservice.append(config.get(sec, option))
	return onto,webpage,webservice



def printResult(vegiObj, mkList):
	"""gibt Ergebnistabelle aus
	"""
	result =  """
Gemuese:                            |  """ + _nonePrint(vegiObj.vegi) + """
================================================================================
Boden:                              |  """ + _kommaList(vegiObj.getGround()) + """
--------------------------------------------------------------------------------
Standort:                           |  """ + _kommaList(vegiObj.getClimate()) + """
--------------------------------------------------------------------------------
Gute Nachbarn:                      |  """ + _kommaList(vegiObj.getGoodNeighbour()) + """
--------------------------------------------------------------------------------
Schlechte Nachbarn:                 |  """ + _kommaList(vegiObj.getBadNeighbour()) + """
--------------------------------------------------------------------------------
Saatzeit drinnen:                   |  """ + _nonePrint(vegiObj.getSeedingTimeIndoor().begin) + """ bis """ + _nonePrint(vegiObj.getSeedingTimeIndoor().end)  + """
--------------------------------------------------------------------------------
Saatzeit draussen:                  |  """ + _nonePrint(vegiObj.getSeedingTimeOutdoor().begin) + """ bis """ + _nonePrint(vegiObj.getSeedingTimeOutdoor().end)  + """
--------------------------------------------------------------------------------
Guenstige Tage zum saeen:          |  """ + _kommaList(mkList) + """
--------------------------------------------------------------------------------
"""
	print result

	
if __name__ == "__main__":
	(vegiName,year) = readInput()
	ontoDict,webPageList,webServiceList = readConfig("vegi.conf")

	vegiObj = vegi.Vegetable(vegiName,ontoDict,webPageList)
	#liest bisher nur die erste URL, die zum Mondkalender. Sollte eigentlich mehrere Dienste einlesen koennen.
	wsObj = webService.WebService(ontoDict,webServiceList)
	mkList = []
	(seedingBegin,seedingEnd) = getMaxSeedingInterval(vegiObj,year)
	if (seedingBegin != None) and (seedingEnd != None):
		mk = processWebService(wsObj,ontoDict,seedingBegin,seedingEnd)
		for moonDay in mk.moonDaysList:
			if goodSeedingDate(moonDay,vegiObj,seedingBegin,seedingEnd):
				mkList.append(moonDay.mcXsdDate)

	printResult(vegiObj,mkList)

	
	
	
	
