#!/usr/bin/python
# -*- coding: iso-8859-1 -*-
import rdflib
import re
		
class Sparql:
	"""Sparql-Anfragen,
	   Basis-Klasse von mehreren Unterklassen
	"""
	def __init__(self,ontoDict):
		"""in self.queryPrefix werden alle fuer die Anfragen benoetigten Praefixe (Namespaces) geschrieben
		"""
		self.queryPrefix = ""
		for key in ontoDict.keys():
			self.queryPrefix = self.queryPrefix + "prefix " + key + ": <" + ontoDict[key] + """#>
"""
	
	def _querySelect1(self, graph, queryString):
		""" fragt nur einen Teil des Tripels ab
		"""
		list=[]
		for result in graph.query(self.queryPrefix + queryString):
			if len(result) > 0:
				try:
					list.append(result[0])
				except IndexError:
					pass
		return list
	
	
	def _querySelect(self, graph, queryString):
		""" gibt Liste in Liste zurck (mehrere Sparql-Ergebnisteile)
		"""
		return graph.query(self.queryPrefix + queryString)


	def _bracket(self, obj):
		"""setzt die Pfadangaben in Sparql-Abfragen in spitze Klammern 
		"""
		if re.search("^<.*>$",obj) != None:
			return obj
		if re.search(":[^/]",obj) != None:
			return obj
		return "<" + obj + ">"
			
###################################################################
###################################################################			
			
class SparqlWebPage(Sparql):
	"""Sparql-Anfragen an die annotierten Web-Seiten
	"""
	def __init__(self, vegi,ontoDict,webpageList):
		"""bekommt die Gemuesesorte, die der Benutzer eingegeben hat
		"""
		Sparql.__init__(self,ontoDict)
		self.links = webpageList
		self.vegiLinks = []
		self._searchForVegi(vegi)
		
		
	def _searchForVegi(self,vegi):
		"""sucht in den Seiten nach dem gefragten Gemuese und speichert die relevanten Seiten in self.vegiLinks
		"""
		for link in self.links:
			qLink = rdflib.ConjunctiveGraph()
			qLink.parse(link)
			for vegiLink in self._linksQuery(qLink,vegi):
				qVegiLink = rdflib.ConjunctiveGraph()
				qVegiLink.parse(vegiLink)
				self.vegiLinks.append([qVegiLink,vegiLink])
			
			
	def _linksQuery(self,link, vegi):
		""" Abfrage fr Gemueseobjekte in den vorgegebenen Links
		"""
		queryString = """SELECT ?obj 
			WHERE { 
			?subj ?pred ?obj	.
			FILTER regex(?obj, '#""" + vegi + """$')
			}"""
		return self._querySelect1(link,queryString)
		
		
	def groundQuery(self):
		"""Abfrage fuer Bodenqualitaet
		"""
		groundList=[]
		for link in self.vegiLinks:
			"""fuer die SPARQL-Abfrage muss der Pfad geaendert werden
			"""
			linkRdf = re.sub("/[^/]*#", "/", link[1]).encode()
			queryString = """SELECT ?ground 
				WHERE { 
				?bag ?pred ?ground .
 				<""" + linkRdf + """>  garden:needsGround ?bag .
 				FILTER regex(?pred, '#_[0-9]+$')
				}"""
			groundList = groundList + self._querySelect1(link[0], queryString)
		return groundList	
		
		
	def climateQuery(self):
		"""Abfrage fuer Standort (sonnig, schattig, etc.)
		"""
		climateList=[]
		for link in self.vegiLinks:
			linkRdf = re.sub("/[^/]*#", "/", link[1]).encode()
			queryString = """SELECT ?climate
				WHERE {
				?bag ?pred ?climate .
				<""" + linkRdf + """> garden:needsClimate ?bag .
				FILTER regex(?pred, '#_[0-9]+$')
				}"""
			climateList = climateList + self._querySelect1(link[0], queryString)
		return climateList
		
		
	def goodNeighbourQuery(self):
		"""Abfrage fuer Mischkultur (gute Nachbarn)
		"""
		neighbourList=[]
		for link in self.vegiLinks:
			linkRdf = re.sub("/[^/]*#", "/", link[1]).encode()
			queryString = """SELECT ?neighbour 
				WHERE {  
				?vegi ?pred ?neighbour .
				<""" + linkRdf + """> garden:inGoodNeighbourhoodWith ?vegi .
				FILTER  (!regex(?neighbour, "22-rdf-syntax-ns#Bag")) 
				}"""
			neighbourList = neighbourList + self._querySelect1(link[0],queryString)
		return neighbourList	
		
		
	def badNeighbourQuery(self):
		"""Abfrage fuer Mischkultur (schlechte Nachbarn)
		"""
		neighbourList=[]
		for link in self.vegiLinks:
			linkRdf = re.sub("/[^/]*#", "/", link[1]).encode()
			queryString = """SELECT ?neighbour 
				WHERE {  
				?vegi ?pred ?neighbour .
				<""" + linkRdf + """> garden:inBadNeighbourhoodWith ?vegi .
				FILTER  (!regex(?neighbour, "22-rdf-syntax-ns#Bag")) 
				}"""
			neighbourList = neighbourList + self._querySelect1(link[0],queryString)
		return neighbourList	
		
		
	def seedingTimeQuery(self, location, timeMargin):
		"""Abfrage fuer die Shzeit
		"""
		timeList=[]
		for link in self.vegiLinks:
			linkRdf = re.sub("/[^/]*#", "/", link[1]).encode()
			queryString = """SELECT ?seedingTime 
				WHERE {
				?vegi """+ timeMargin +""" ?seedingTime .
				<""" +linkRdf + """> """ + location + """ ?vegi . 
				}"""
			timeList = timeList + self._querySelect1(link[0],queryString)
		return timeList
		
		
	def partOfMonthQuery(self, seedingTime):
		"""Abfrage, um welchen Teil des Monats es sich bei einer Saehzeit handelt: Anfang, Mitte oder Ende des Monats
		"""		
		partOfMonthList = []
		for link in self.vegiLinks:
			linkRdf = re.sub("/[^/]*#", "/", link[1]).encode()
			queryString = """SELECT  ?inst
				WHERE {
				""" + seedingTime + """ rdf:type  ?inst
				}"""	
			partOfMonthList=partOfMonthList+self._querySelect1(link[0],queryString)

		return partOfMonthList	
			
			
	def monthDateTimeQuery(self, seedingTime):
		"""Abfrage, um welchen Monat es sich bei einer Saehzeit handelt
		"""
		monthList = []
		for link in self.vegiLinks:
			linkRdf = re.sub("/[^/]*#", "/", link[1]).encode()
			queryString = """SELECT  ?month
				WHERE {
				""" + self._bracket(seedingTime) + """ time:inDateTime  ?month
				}"""
			monthList = monthList + self._querySelect1(link[0],queryString)
		return monthList	
			
			
	def typeQuery(self, obj=""):
		"""bestimmt die Klassenzugehrigkeit des Gemueses
		"""	
		typeList=[]
		for link in self.vegiLinks:
			linkRdf = re.sub("/[^/]*#.*$", "/", link[1]).encode()
			queryString = """SELECT ?class 
				WHERE { 
				""" + self._bracket(linkRdf + obj) + """ rdf:type ?class
				}"""
			typeList = typeList + self._querySelect1(link[0],queryString)
		return typeList	
	

###################################################################
###################################################################				
class SparqlWebService(Sparql):
	"""Sparql-Anfragen an die OWLS-Ontologie ueber den Web-Service
	"""
	
	def __init__(self, ontoDict,webserviceList):
		Sparql.__init__(self,ontoDict)
		self.urlWebService = rdflib.ConjunctiveGraph()
		self.urlWebService.parse(webserviceList[0])


	def atomicProcessQuery(self):
		"""Abfrage nach einem Atomic Process
		"""		
		queryString = """SELECT ?atomicPr  
			WHERE  { 
 			?atomicPr ?pred grounding:WsdlAtomicProcessGrounding 
  			}"""
		return self._querySelect1(self.urlWebService, queryString)
	
	
	def wsdlDocQuery(self, atomicProc):
		"""Abfrage nach der Adresse des WSDL-Dokuments
		"""			
		queryString = """SELECT ?wsdlDoc  
			WHERE  { 
 			""" + atomicProc +""" grounding:wsdlDocument ?wsdlDoc 
  			}"""
		return self._querySelect1(self.urlWebService, queryString)
	
	
	def wsdlServiceQuery(self, atomicProc):
		"""Abfrage nach der Adresse des Web Services
		"""		
		queryString = """SELECT ?wsdlService  
			WHERE  { 
 			""" + atomicProc +""" grounding:wsdlService ?wsdlService 
  			}"""
		return self._querySelect1(self.urlWebService, queryString)
	
	
	def portTypeQuery(self, atomicProc):
		"""Abfrage nach dem Port-Type
		"""		
		queryString = """SELECT ?pt 
			WHERE  { 
 			?wsdlOpRef grounding:portType ?pt .
 			""" + atomicProc + """ grounding:wsdlOperation ?wsdlOpRef 
  			}"""
		return self._querySelect1(self.urlWebService, queryString)


	def operationQuery(self, atomicProc):
		"""Abfrage nach der Operation
		"""		
		queryString = """SELECT ?op 
			WHERE  { 
 			?wsdlOpRef grounding:operation ?op .
 			""" + atomicProc + """ grounding:wsdlOperation ?wsdlOpRef 
  			}"""
		return self._querySelect1(self.urlWebService, queryString)
	
	
	def inputQuery(self, atomicProc):
		queryString = """SELECT ?wsdlMessageMap ?para ?monat 
			WHERE  { 
			?wsdlMessageMap ?para ?monat .
 			""" + atomicProc + """ grounding:wsdlInput ?wsdlMessageMap 
  			}"""
		return self._querySelect(self.urlWebService, queryString)	
	
	
	def outputQuery(self, atomicProc):
		queryString = """SELECT ?wsdlMessageMap ?para ?monat 
			WHERE  { 
			?wsdlMessageMap ?para ?monat .
 			""" + atomicProc + """ grounding:wsdlOutput ?wsdlMessageMap 
  			}"""
		return self._querySelect(self.urlWebService, queryString)
	
	
#################################################################
#################################################################
class SparqlMoonCalendar(Sparql):
	"""Sparql-Anfragen an den annotierten Mondkalender
	"""
	def __init__(self, ontoDict,rdfUrl):
		Sparql.__init__(self,ontoDict)
		self.onto = rdflib.ConjunctiveGraph()
		self.onto.parse(rdfUrl)
		
		
	def signOfZodiacQuery(self, moonCalendarDate):
		"""fragt das Sternzeichen eines Kalendertages ab
		"""
		queryString = """SELECT  ?class
			WHERE {
			?sign rdf:type ?class	.
			<""" + moonCalendarDate + """> moon:isInSignOfZodiac  ?sign
			}"""
		return self._querySelect1(self.onto,queryString)
	
	
	def ageOfMoonQuery(self, moonCalendarDate):
		"""fragt die Mondphase eines Kalendertages ab
		"""
		queryString = """SELECT  ?dec
			WHERE {
			<""" + moonCalendarDate + """> moon:hasAgeOfMoonInXsdDecimal  ?dec
			}"""
		return self._querySelect1(self.onto,queryString)
	
	
	def mcDateQuery(self, moonCalendarDate):
		"""fragt das Datum als xsd:date ab
		"""
		queryString = """SELECT  ?date
			WHERE {
			<""" + moonCalendarDate + """> moon:inXSDDateTime  ?date
			}"""
		return self._querySelect1(self.onto,queryString)
	
	
	def mcOneDayQuery(self):
		"""sucht eine rdf:Description, die ein neues Datum liefert, heraus
		"""
		queryString = """SELECT  ?date 
			WHERE {
			?date moon:Date  ?obj
			}"""
		return self._querySelect1(self.onto,queryString)
	
	
	
	
#################################################################
#################################################################
class SparqlGardenOntology(Sparql):
	"""Sparql-Anfragen an die Garten-Ontologie
	"""
	
	def __init__(self, ontoDict):
		Sparql.__init__(self,ontoDict)
		self.onto = rdflib.ConjunctiveGraph()
		self.onto.parse(ontoDict["garden"])
		
		
	def vegiSuperClassQuery(self, vegi):
		"""ermittelt die Oberklasse des Gemueses: entweder LeafVegetable, FruitVegetable, RootVegetable oder BlossomVegetable (sind Unterklassen von Vegetable)
		"""
		if vegi == "":
			return ""
		vegiClass = vegi
		queryString = """SELECT  ?superclass1
			WHERE { 
			<""" + vegi + """> rdfs:subClassOf ?superclass1	
			}"""
		for  vegiClass in self._querySelect1(self.onto,queryString):
			if re.search("#Vegetable$",str(vegiClass)) != None:
				return vegi
			if vegiClass != "":
				tmp = self.vegiSuperClassQuery(str(vegiClass))
				if tmp != "":
					return tmp
		return ""
		
		
	def bestSeedingTimeSignOfZodiacQuery(self, superClass):
		"""ermittelt die Sternzeichen, bei denen das Gemuese am besten gesaeht wird. superClass ist die Oberklasse des Gemueses (Leaf-, Root, Fruit- oder BlossomVegatable) die die Property garden:bestSeedingTime hat.Dieser Umweg ist notwendig, da Sparql keine Vererbung unterstuetzt
		"""	
		signOfList = []
		if superClass == "":
			return []
		queryString = """SELECT  ?sign1 ?sign2 ?sign3
			WHERE { 
			?anonymCl4 rdf:first ?sign3	.
			?anonymCl3 rdf:rest ?anonymCl4	.
			?anonymCl3 rdf:first ?sign2	.
			?parseType2 rdf:rest ?anonymCl3	.
			?parseType2 rdf:first ?sign1	.	
			?anonymCl2 owl:unionOf ?parseType2 .
			?anonymCl rdf:first ?anonymCl2	.
			?parseType1 rdf:rest ?anonymCl	.
			?class owl:intersectionOf ?parseType1	.
			?anonymRestr owl:someValuesFrom ?class	.
			<""" + superClass + """> rdfs:subClassOf ?anonymRestr	
			}"""
		for queryTupple in self._querySelect(self.onto, queryString):
			for t in queryTupple:
				signOfList.append(t)
		return signOfList


	
		
		
	def bestSeedingTimeMoonPhaseQuery(self, superClass):
		"""ermittelt die Mondphase, bei der das Gemuese am besten gesaeht wird. superClass ist die Oberklasse des Gemueses (Leaf-, Root, Fruit- oder BlossomVegatable) die die Property garden:bestSeedingTime hat.Dieser Umweg ist notwendig, da Sparql keine Vererbung unterstuetzt
		"""			
		if superClass == "":
			return []
		queryString = """SELECT  ?moon
			WHERE { 
			?superclass4 rdf:first ?moon .
			?superclass3 owl:intersectionOf ?superclass4 .
			?superclass2 owl:someValuesFrom ?superclass3 .
			?superclass2  owl:onProperty  garden:bestSeedingTime .
			<""" + superClass + """> ?pred1 ?superclass2  
		   }"""
		return self._querySelect1(self.onto,queryString)
		
		
	def gMonthQuery(self, month):
		"""Abfrage nach dem konkreten Monat eines Anfangs- oder Endzeitpunktes in Form eines gMonth
		"""
		gMonthList = []
		queryString = """SELECT  ?gMonth
			WHERE {
			?anonymCl owl:hasValue ?gMonth	.
			""" + self._bracket(month) + """ rdfs:subClassOf ?anonymCl	.
			FILTER isLiteral(?gMonth)
			}"""
		gMonthList = gMonthList + self._querySelect1(self.onto,queryString)
		return gMonthList
					
	
	def gDayQuery(self, instant, beginOrEnd):
		"""Abfrage nach dem konkreten Tag eines Anfangs- oder Endzeitpunktes in Form eines gDay
		"""		
		gDayList = []
		queryString = """SELECT  ?day
			WHERE {
			?date time:day ?day	.
			?DateTime owl:hasValue ?date	.
			?DateTime owl:onProperty time:inDateTime	.
			?instant rdfs:subClassOf ?DateTime	.
			?intervalRest owl:someValuesFrom ?instant	.
			?intervalRest owl:onProperty """ + beginOrEnd + """	. 
			?interval rdfs:subClassOf ?intervalRest	. 
			?obj owl:someValuesFrom ?interval .
			?obj owl:onProperty garden:isInside	.
			""" + self._bracket(instant) + """ rdfs:subClassOf ?obj
			}"""
		gDayList = gDayList + self._querySelect1(self.onto,queryString)
		return gDayList
	
	
		
		
