#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

import sparql
import sys

#TODO: Fehlerbehandlung!


class Vegetable:
	"""Informant fuer Gemueseeigenschaften
	"""
	def __init__(self, vegi,ontoDict,webpageList):
		"""Konstruktor fuer eine bestimmte Gemuesesorte, ueber die Informationen eingeholt werden sollen
		"""
		self.vegi = vegi
		self.q = sparql.SparqlWebPage(vegi,ontoDict,webpageList)
		self.qGarden = sparql.SparqlGardenOntology(ontoDict)
		self.ground = []
		self.climate = []
		self.indoor = SeedingTime()
		self.outdoor = SeedingTime() 
		self.goodNeighbour = []
		self.badNeighbour = []
		self.classType = ""
		self.setGround()
		self.setClimate()
		self.setSeedingTimeIndoor()
		self.setSeedingTimeOutdoor()
		self.setGoodNeighbour()
		self.setBadNeighbour()
		self.setType()
		self.setBestSeedingTimeSignOfZodiac()
		self.setBestSeedingTimeMoonPhase()
		
	def setGround(self):
		"""holt sich Informationen ueber die notwendige Bodenbeschaffenheit des Gemueses.
		"""
		for ground in self.q.groundQuery():
			self.ground.append(str(ground).split("/")[-1])

			
	def getGround(self):
		"""gibt eine Liste von Bodeneigenschaften zurueck.
		"""
		return self.ground
	
	def setClimate(self):
		"""holt sich Informationen ueber den notwendigen Standort des Gemueses
		"""		
		for climate in self.q.climateQuery():
			self.climate.append(str(climate).split("/")[-1])
			
	def getClimate(self):
		"""gibt eine Liste von Klimaeigenschaften zurueck.
		"""
		return self.climate
	
	def setGoodNeighbour(self):
		"""holt sich Informationen ueber gute Nachbarpflanzen fuer das Gemuese
		"""
		for neighbour in self.q.goodNeighbourQuery():
			try:
				self.goodNeighbour.append(str(neighbour).split("/")[-1])
			except IndexError:
				pass

	def getGoodNeighbour(self):
		return self.goodNeighbour	
	
	def setBadNeighbour(self):
		"""holt sich Informationen ueber schlecht Nachbarpflanzen fuer das Gemuese
		"""
		for neighbour in self.q.badNeighbourQuery():
			try:
				self.badNeighbour.append(str(neighbour).split("/")[-1])
			except IndexError:
				pass
			
	def getBadNeighbour(self):
		return self.badNeighbour	
	
	def setSeedingTimeIndoor(self):
		"""holt sich Informationen ueber die Saatzeit des Gemueses in Innenraeumen (Treibhaus)
		"""
		try:
			self.indoor.begin = str(self.q.seedingTimeQuery("garden:isSeededIndoor", "time:hasBeginning")[0])
			self.indoor.beginGMonthDay = str(self.getGMonthOfMonth(str(self.q.monthDateTimeQuery(str(self.indoor.begin))[0])))
			self.indoor.begin = self.indoor.begin.split("/")[-1]
			self.indoor.beginGMonthDay = self.indoor.beginGMonthDay + str(self.getGDayFromBeginOrEnd(str(self.q.typeQuery(self.indoor.begin)[0]),"time:hasBeginning")).replace("--","")
			self.indoor.end = str(self.q.seedingTimeQuery("garden:isSeededIndoor", "time:hasEnd")[0])
			self.indoor.endGMonthDay = str(self.getGMonthOfMonth(str(self.q.monthDateTimeQuery(str(self.indoor.end))[0])))
			self.indoor.end = self.indoor.end.split("/")[-1]	
			self.indoor.endGMonthDay = self.indoor.endGMonthDay + str(self.getGDayFromBeginOrEnd(str(self.q.typeQuery(self.indoor.end)[0]),"time:hasEnd")).replace("--","")
		except IndexError:
			pass
			
	def getSeedingTimeIndoor(self):
		return self.indoor			
			
	def setSeedingTimeOutdoor(self):
		"""holt sich Informationen ueber die Saatzeit des Gemueses im Freiland
		"""
		try:
			self.outdoor.begin = str(self.q.seedingTimeQuery("garden:isSeededOutdoor", "time:hasBeginning")[0])
			self.outdoor.beginGMonthDay = str(self.getGMonthOfMonth(str(self.q.monthDateTimeQuery(str(self.outdoor.begin))[0])))
			self.outdoor.begin = self.outdoor.begin.split("/")[-1]
			self.outdoor.beginGMonthDay = self.outdoor.beginGMonthDay + str(self.getGDayFromBeginOrEnd(str(self.q.typeQuery(self.outdoor.begin)[0]),"time:hasBeginning")).replace("--","")
			self.outdoor.end = str(self.q.seedingTimeQuery("garden:isSeededOutdoor", "time:hasEnd")[0])
			self.outdoor.endGMonthDay = str(self.getGMonthOfMonth(str(self.q.monthDateTimeQuery(str(self.outdoor.end))[0])))
			self.outdoor.end = self.outdoor.end.split("/")[-1]
			self.outdoor.endGMonthDay = self.outdoor.endGMonthDay + str(self.getGDayFromBeginOrEnd(str(self.q.typeQuery(self.outdoor.end)[0]),"time:hasEnd")).replace("--","")
		except IndexError:
			pass
		
	def getSeedingTimeOutdoor(self):
		return self.outdoor
	
	def setMonthDateTime(self, anySeedingTime):
		"""anySeedingTime ist das Ergebnis der setSeedingTimeIndoor oder 
		setSeedingTimeOutdoor und hat die Form: file///.../EndeFebruar. 
		Die Funktion gibt den Monat von anySeedingTime zurueck, hier: February
		"""
		try:
			self.monthDateTime = str(self.q.monthDateTimeQuery(anySeedingTime)[0])
		except IndexError:
			self.monthDateTime = ""
	
	def getMonthDateTime(self):
		return self.monthDateTime
	
	def setPartOfMonth(self, anySeedingTime):
		"""anySeedingTime ist das Ergebnis der Funktion setSeedingTimeIndoor oder setSeedingTimeOutdoor und hat z.B. die Form: file///.../EndeFebruar. Die Funktion gibt den Monatsteil von anySeedingTime zurueck, hier: InstantThirdPartOfMonth
		"""		
		try:
			self.partOfMonth = str(self.q.partOfMonthQuery(anySeedingTime)[0])
		except IndexError:
			self.partOfMonth
		
	def getPartOfMonth(self):
		return self.partOfMonth
	
	
	def getGMonthOfMonth(self, month):
		"""month ist das Ergebnis von setMonthDateTime, z.B. February; gibt den entsprechenden gMonth zurueck, z.B. --02
		"""
		try:
			return str(self.qGarden.gMonthQuery(month)[0])
		except IndexError:
			return ""
		
	def getGDayFromBeginOrEnd(self, instant, beginOrEnd):
		"""erstes Attribut: instant ist das Ergebnis von setPartOfMonth, z.B. InstantThirdPartOfMonth. 
		zweites Attribut: beginOrEnd gibt an, ob der Anfangs- oder Endzeitpunkt zurueck gegeben werden soll. 
		Die Funktion liefert das entsprechende gDay, z.B. ---18.
		"""
		try:
			return str(self.qGarden.gDayQuery(instant, beginOrEnd)[0])
		except IndexError:
			return ""
		
	
	def setType(self):
		"""holt sich Informationen ueber die Klassenzugehoerigkeit des Gemueses
		"""	
		#TODO ueberpruefung ob die Klasse wirklich existiert oder ob es einen annotationsfehler gibt
		try:
			self.classType = str(self.q.typeQuery(self.vegi)[0])
		except IndexError:
			self.classType = ""
			
	def getType(self):
		return self.classType
	
	
	def setBestSeedingTimeSignOfZodiac(self):
		"""holt sich Informationen ueber die Sternzeichen, in denen am besten das Gemuese gesaeht wird
		"""			
		superClass = self.qGarden.vegiSuperClassQuery(self.classType)
		self.bestSigns = self.qGarden.bestSeedingTimeSignOfZodiacQuery(superClass)

		
	def getBestSeedingTimeSignOfZodiac(self):
		return self.bestSigns
	
	
	
	def setBestSeedingTimeMoonPhase(self):
		"""holt sich Informationen ueber die Mondphase, in der am besten das Gemuese gesaeht wird
		"""			
		try:
			superClass = self.qGarden.vegiSuperClassQuery(self.classType)
			self.bestPhase = str(self.qGarden.bestSeedingTimeMoonPhaseQuery(superClass)[0])
		except IndexError:
			self.bestPhase = ""
		
	def getBestSeedingTimeMoonPhase(self):
		return self.bestPhase
	
		
class SeedingTime:
	"""enthaelt ein Saatintervall in unterschiedlichen Formaten (GMonthDay,Sprachlicher String)
	"""
	def __init__(self, begin=None, end=None, beginGMonthDay=None, endGMonthDay=None):
		self.begin = begin
		self.end = end
		self.beginGMonthDay = beginGMonthDay
		self.endGMonthDay = endGMonthDay

