#!/usr/bin/python
# -*- coding: iso-8859-1 -*-
import sparql
import re
from ZSI.wstools import WSDLTools
import httplib
import os
from tempfile import NamedTemporaryFile	
from time import *		

class WebService:
	"""Informant fuer den Web Service
	"""
	def __init__(self, ontoDict,urlWebService):
		"""Konstruktor fuer einen bestimmten Web Service, ueber den Informationen eingeholt werden sollen. Das Objekt wird mit den Daten einer OWLS-Datei initalisiert.
		"""	
		self.sparql = sparql.SparqlWebService(ontoDict,urlWebService)
		self.atomicProc = ""
		self.inputWs = {}
		self.outputWs = {}
		self.wsdlDoc = ""
		self.wsdlService = ""
		self.portType = ""
		self.operation = ""
		self.rdfFile = None
		self.setAtomicProc()
		self.setInput()
		self.setOutput()
		self.setOperation()
		self.setPortType()
		self.setWsdlDoc()
		self.setWsdlService()
		self.wsdl = WebServiceWsdl(self.getWsdlDoc(),self.getWsdlService(),self.getPortType(),self.getOperation())

	
	
	def setInput(self):
		"""durchlaeuft Ergebnisliste von inputQuery. Jedes Element der Ergebnisliste ist eine Liste mit drei Elementen: anonyme Klasse, Property, Wert. Das Tripel wird als Dic in einem Dic repraesentiert {anonymeKlasse, {Property, Wert}}.
		"""
		for q in self.sparql.inputQuery(self.atomicProc):
			if not self.inputWs.has_key(str(q[0])):
				self.inputWs[str(q[0])] = {}	
			self.inputWs[str(q[0])] [str(q[1])] = q[2]	


	def getInput(self):
		return self.inputWs
			
			
	def setOutput(self):
		"""durchlaeuft Ergebnisliste von outputQuery. Jedes Element der Ergebnisliste ist eine Liste mit drei Elementen: anonyme Klasse, Property, Wert. Das Tripel wird als Dic in einem Dic repraesentiert {anonymeKlasse, {Property, Wert}}.
		"""
		for q in self.sparql.outputQuery(self.atomicProc):
			if not self.outputWs.has_key(str(q[0])):
				self.outputWs[str(q[0])] = {}	
			self.outputWs[str(q[0])] [str(q[1])] = q[2]	


	def getOutput(self):
		return self.outputWs
			
			
	def setAtomicProc(self):
		self.atomicProc = "<"+str(self.sparql.atomicProcessQuery()[0])+">"


	def getAtomicProc(self):
		return self.atomicProc
	
	
	def setWsdlDoc(self):
		self.wsdlDoc = str(self.sparql.wsdlDocQuery(self.atomicProc)[0])
		
	def getWsdlDoc(self):
		return self.wsdlDoc
		
		
	def setWsdlService(self):
		self.wsdlService =  str(self.sparql.wsdlServiceQuery(self.atomicProc)[0]).split("#")[1]

	def getWsdlService(self):
		return self.wsdlService
	

	def setPortType(self):
		self.portType = str(self.sparql.portTypeQuery(self.atomicProc)[0]).split("#")[1]
		
	def getPortType(self):
		return self.portType


	def setOperation(self):
		self.operation = str(self.sparql.operationQuery(self.atomicProc)[0]).split("#")[1]
		
	def getOperation(self):
		return self.operation

	def processXSLT(self,html):
		"""schreibt einen Transformationsstring in eine temporaere Datei (XSLT-Code). Aufruf von xsltproc --html mit der tmp Datei als Parameter. xsltproc bekommt den zu transformierenden String (HTML-Doc) ueber seine Standardeingabe und gibt den XSL-Ergebnisbaum (RDF-Doc) ueber seine Standardausausgabe zurueck.
		"""
		xsltCode = ""
		for outMessage in self.getOutput().values():
			for key in outMessage.keys():
				if re.search('#xsltTransformationString$',key):
					xsltCode = outMessage[key]
		if xsltCode == "":
			return
		tmp = NamedTemporaryFile()
		tmp.write(xsltCode.strip())
		tmp.flush()
		(inFile,outFile) = os.popen2("xsltproc --html " + tmp.name + " - 2> /dev/null")
		inFile.write(html)
		inFile.close()
		if self.rdfFile != None:
			self.rdfFile.close()
		self.rdfFile = NamedTemporaryFile()
		test = outFile.read()
		self.rdfFile.write(str(test))
		self.rdfFile.flush()
		outFile.close()
		tmp.close()
		
#######################################################################
#######################################################################	
	
class WebServiceWsdl:
	"""Nutzung eines Web Services anhand eines WSDL-Dokuments. Es wird nur http:adress, http-get-Methode und URL-Replacement untersteutzt.
	"""
	def __init__(self,url,service,port,operation):
		"""Konstuktor Initalisiert das Object mit den Daten aus der WSDL-Datei (url) und den owls Daten (service,port,operation)
		"""
		self.lastResponse = ""
		self.url = url
		self.reader = WSDLTools.WSDLReader()
		if (re.match("^http://",url,re.I)):
			self.wsdl = self.reader.loadFromURL(url)
		else:
			self.wsdl = self.reader.loadFromFile(re.sub("^[Ff][Ii][Ll][Ee]://","/",url))
		self.port = self.wsdl.services[service].ports[port]
		self.httpAddressLocation = None
		if isinstance(self.port.getAddressBinding(),WSDLTools.HttpAddressBinding):
			self.httpAddressLocation = self.port.getAddressBinding().location

		self.httpVerb =  self.port.getBinding().findBinding(WSDLTools.HttpBinding).verb
		self.httpOperationLocation =  self.port.getBinding().operations[operation].findBinding(WSDLTools.HttpOperationBinding).location
		self.replacement = self.port.getBinding().operations[operation].input.findBinding(WSDLTools.HttpUrlReplacementBinding)
		self.xmlout = self.port.getBinding().operations[operation].output.findBinding(WSDLTools.MimeXmlBinding)

	def processWebservice(self, parameterDict):
		"""nutzt einen Web Service mit atomarem Prozess. 
		"""
		if self._checkService():
			conn = httplib.HTTPConnection(re.sub("[hH][tT][tT][pP]://","",self.httpAddressLocation))
			conn.request(self.httpVerb, self._parameterReplace(parameterDict))
			stat = "Anfrage des Webservices fuer: "
			for key in parameterDict.keys():
				stat = stat + key + ": " + parameterDict[key] + "   "
			print stat
			response = conn.getresponse()
			if response.status == 200:
				self.lastResponse = response.read()
				return True
		return False
	
	def _checkService(self):
		"""ueberprueft, ob alle benoetigten Angaben und Methoden aus dem WSDL-Dokument erhaeltlich sind.
		"""
		if ((self.httpAddressLocation) and (self.httpVerb) and (re.match("get",self.httpVerb,re.I)) and (self.httpOperationLocation) and (self.replacement)):
			return True
		return False

	def _parameterReplace(self,parameterDict):
		"""die Keys des parameterDict entsprechen den URL-Replacement-Platzhaltern und werden mit den Values ersetzt, beim Mondkalender z.B. (month):April, (year):2007
		"""
		op = self.httpOperationLocation
		for key in parameterDict.keys():
			op = re.sub("\("+key+"\)",parameterDict[key],op)
		return op
